﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace TestAnimation
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }
        /// <summary>
        /// 跑动的点里的path
        /// </summary>
        private Path m_PathPoint;
        /// <summary>
        /// 故事版
        /// </summary>
        private Storyboard m_Sb;

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            m_PathPoint = (Path)FindResource("path_Point");
            m_Sb = new Storyboard();
            btn_Init_Click(null, null);
        }

        private void btn_Init_Click(object sender, RoutedEventArgs e)
        {
            int count = 1;
            if (int.TryParse(tbx_Count.Text, out count))
            {
                Init(count);
                m_Sb.Begin(this);
            }
        }



        private void Init(int count)
        {
            cvs.Children.Clear();
            m_Sb.Children.Clear();
            Random rd = new Random();
            for (int i = 0; i < count; i++)
            {
                //颜色
                byte r = (byte)rd.Next(0, 255);
                byte g = (byte)rd.Next(0, 255);
                byte b = (byte)rd.Next(0, 255);

                //运动轨迹
                Path particlePath = GetParticlePath(new Point(100, 20 + i * 10), new Point(300, -20 + i * 10), new Point(500, 20 + i * 10), new byte[] { r, g, b });
                cvs.Children.Add(particlePath);

                // 跑动的点
                Grid grid = GetRunPoint(new byte[] { r, g, b });
                AddPointToStoryboard(grid, m_Sb, particlePath);
                Canvas.SetLeft(grid, -grid.Width / 2);
                Canvas.SetTop(grid, -grid.Height / 2);
                cvs.Children.Add(grid);
            }
        }
        #region 获取运动轨迹
        /// <summary>
        /// 获取运动轨迹
        /// </summary>
        /// <param name="startPoint">起点</param>
        /// <param name="controlPoint">控制点</param>
        /// <param name="endPoint">终点</param>
        /// <param name="rgb">颜色:r,g,b</param>
        /// <returns>Path</returns>
        private Path GetParticlePath(Point startPoint, Point controlPoint, Point endPoint, byte[] rgb)
        {
            Path path = new Path();
            PathGeometry pg = new PathGeometry();
            PathFigure pf = new PathFigure();
            pf.StartPoint = startPoint;
            QuadraticBezierSegment qbs = new QuadraticBezierSegment();
            qbs.Point1 = controlPoint;
            qbs.Point2 = endPoint;
            pf.Segments.Add(qbs);
            pg.Figures.Add(pf);
            path.Data = pg;
            path.Stroke = new SolidColorBrush(Color.FromArgb(255, rgb[0], rgb[1], rgb[2]));
            path.Stretch = Stretch.None;
            return path;
        }
        #endregion
        #region 获取跑动的点
        /// <summary>
        /// 获取跑动的点
        /// </summary>
        /// <param name="rgb">颜色:r,g,b</param>
        /// <returns>Grid</returns>
        private Grid GetRunPoint(byte[] rgb)
        {
            //一个Grid里包含一个椭圆 一个Path 椭圆做阴影
            //Grid
            Grid grid = new Grid();
            grid.Width = 40;
            grid.Height = 15;
            grid.RenderTransformOrigin = new Point(0.5, 0.5);
            //Ellipse
            Ellipse ell = new Ellipse();
            ell.Width = 40;
            ell.Height = 15;
            ell.Fill = new SolidColorBrush(Color.FromArgb(55, rgb[0], rgb[1], rgb[2]));
            grid.Children.Add(ell);
            //Path
            Path path = new Path();
            path.Data = m_PathPoint.Data;
            path.Width = m_PathPoint.Width;
            path.Height = m_PathPoint.Height;
            LinearGradientBrush lgb = new LinearGradientBrush();
            lgb.StartPoint = new Point(0, 0);
            lgb.EndPoint = new Point(1, 0);
            lgb.GradientStops.Add(new GradientStop(Color.FromArgb(88, rgb[0], rgb[1], rgb[2]), 0));
            lgb.GradientStops.Add(new GradientStop(Color.FromArgb(255, 255, 255, 255), 1));
            path.Fill = lgb;
            path.Stretch = Stretch.Fill;
            grid.Children.Add(path);
            return grid;
        }
        #endregion
        #region 将点加入到动画故事版
        /// <summary>
        /// 将点加入到动画故事版
        /// </summary>
        /// <param name="point">运动的点</param>
        /// <param name="sb">故事版</param>
        /// <param name="particlePath">运动轨迹</param>
        private void AddPointToStoryboard(Grid point, Storyboard sb, Path particlePath)
        {
            MatrixTransform matrix = new MatrixTransform();
            point.RenderTransform = matrix;
            string name = "m" + Guid.NewGuid().ToString().Replace("-", "");
            this.RegisterName(name, matrix);

            MatrixAnimationUsingPath matrixAnimation = new MatrixAnimationUsingPath();
            matrixAnimation.PathGeometry = particlePath.Data.GetFlattenedPathGeometry();
            matrixAnimation.Duration = new Duration(TimeSpan.FromSeconds(5));
            matrixAnimation.RepeatBehavior = RepeatBehavior.Forever;
            matrixAnimation.AutoReverse = false;
            matrixAnimation.IsOffsetCumulative = false;
            matrixAnimation.DoesRotateWithTangent = true;//沿切线旋转

            Storyboard.SetTargetName(matrixAnimation, name);
            Storyboard.SetTargetProperty(matrixAnimation, new PropertyPath(MatrixTransform.MatrixProperty));
            sb.Children.Add(matrixAnimation);
        }
        #endregion
    }
}
